package jalview.gui;

import jalview.jbgui.GFinder;
import jalview.datamodel.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;

public class Finder extends GFinder
{
  AlignViewport av;
  AlignmentPanel ap;
  JInternalFrame frame;
  String searchString;
  SuperGroup searchGroup;

  int seqIndex = 0;
  int resIndex = 0;
  public Finder(AlignViewport av, AlignmentPanel ap, JInternalFrame f)
  {
    this.av = av;
    this.ap = ap;
    frame = f;

    // all a big pain, but we need to wait until the frame is visible before the textfield can
    // obtain the focus/////////////////////////
    frame.addInternalFrameListener(new InternalFrameAdapter()
    {
      public void internalFrameOpened(InternalFrameEvent evt)
      {
        SwingUtilities.invokeLater(new Runnable()
        {
          public void run()
          {   textfield.requestFocus();      }
       });
     }

     public void internalFrameClosing(InternalFrameEvent evt)
     {
       cancel_actionPerformed(null);
     }
});

  }

  public void textfield_actionPerformed(ActionEvent e)
  {
    doSearch(false);
  }

  public void findNext_actionPerformed(ActionEvent e)
  {
    doSearch(false);
  }

  public void findAll_actionPerformed(ActionEvent e)
  {
    resIndex=0;
    seqIndex=0;
    doSearch(true);
  }

  public void cancel_actionPerformed(ActionEvent e)
  {
    try{
      // if allResults is null, this effectively switches displaySearch flag in seqCanvas
      ap.highlightSearchResults( null );
     // frame.setClosed(true);
    }catch(Exception ex){ }
  }


  public void createNewGroup_actionPerformed(ActionEvent e)
  {
    for(int i=0; i<searchGroup.getSize(); i++)
      av.alignment.addGroup( searchGroup.getGroupeAt(i));

    ap.av.alignment.addSuperGroup(searchGroup);
    ap.highlightSearchResults( null );
  }


  void doSearch(boolean findAll)
  {
    createNewGroup.setEnabled(false);
    searchString = textfield.getText().toUpperCase();

    ArrayList searchResults = new ArrayList();
    int [] allResults = null;

    Sequence seq;
    String item=null;
    boolean found = false;

    /// is the searchString a residue number?
    try{
      int res = Integer.parseInt(searchString);
      found = true;

      if(av.getSelection().size()>0)
        seq = (Sequence)(av.getSelection().sequenceAt(0));
      else
        seq = (Sequence)av.getAlignment().getSequenceAt(0);

      searchResults.add( Integer.toString( av.getAlignment().findIndex(seq) ) );
      searchResults.add( Integer.toString( seq.findIndex(res)-1 ) );
      searchResults.add( Integer.toString( seq.findIndex(res)-1 ) );

    }catch(NumberFormatException ex){}

    jalview.schemes.UserColourScheme ucs = new jalview.schemes.UserColourScheme();
    Color [] newColors = new Color[24];
    for(int i=0; i<24; i++)
      newColors[i] = new Color(60,160,115);

    ucs.setColourScheme(newColors);

    searchGroup = new SuperGroup(searchString, ucs , true, true, false);

    int end = av.alignment.getHeight();


    SequenceGroup rubberband = av.getRubberbandGroup();
    if(rubberband!=null)
      end = rubberband.getEndRes();



    while( !found && seqIndex<end)
    {
      seq = (Sequence)av.getAlignment().getSequenceAt(seqIndex);

      if(   av.getSelection().size()>0 && !av.getSelection().contains(seq)
            || (rubberband!=null && !rubberband.sequences.contains(seq))  )
      {
        // if a selection has been made, only search within that selection
        seqIndex++;
        resIndex=0;
        continue;
      }

      item = seq.getSequence();

      if(rubberband!=null)
        item = item.substring(0, rubberband.getEndRes()+1);

      ///Shall we ignore gaps????
      StringBuffer noGaps = new StringBuffer();
      int insertCount=0;
      ArrayList spaces = new ArrayList();

      for (int j=0; j < item.length(); j++)
      {

        if(!jalview.util.Comparison.isGap(item.charAt(j)))
         {
           noGaps.append(item.charAt(j));
           spaces.add(new Integer(insertCount));
         }
        else
           insertCount++;
      }


      com.stevesoft.pat.Regex regex = new  com.stevesoft.pat.Regex(searchString);

      for(int r = resIndex; r<noGaps.length(); r++)
      {

       if( regex.searchFrom( noGaps.toString(), r ) )
       {
         resIndex = regex.matchedFrom();


         if(rubberband!=null && ( resIndex+ Integer.parseInt(spaces.get(resIndex).toString()) )<rubberband.getStartRes())
           continue;

         searchResults.add( Integer.toString( seqIndex) );
         searchResults.add( Integer.toString( resIndex+ Integer.parseInt(spaces.get(resIndex).toString()) ) );
         searchResults.add( Integer.toString( regex.matchedTo()-1 + Integer.parseInt(spaces.get(regex.matchedTo()-1).toString()) ) );

         SequenceGroup sg = new SequenceGroup(searchString, ucs, true, true, false,
                                             resIndex+ Integer.parseInt(spaces.get(resIndex).toString()) ,
                                      regex.matchedTo()-1 + Integer.parseInt(spaces.get(regex.matchedTo()-1).toString())
            );
         sg.addSequence((Sequence)av.getAlignment().getSequenceAt(seqIndex));
         searchGroup.addGroup( sg );

         if(!findAll)
         {
           // thats enough, break and display the result
           found = true;
           resIndex++;
           break;
         }

         r=resIndex;
       }
      }
      if(!found)
      {
        seqIndex++;
        resIndex = 0;
      }
    }

    if(searchResults.size()>0)
    {
      allResults = new int[searchResults.size()];
      for(int i=0; i<searchResults.size(); i++)
        allResults[i] = Integer.parseInt(searchResults.get(i).toString());

      createNewGroup.setEnabled(true);
    }
    else
    {
      JOptionPane.showInternalMessageDialog(this, "Finished searching", null, JOptionPane.INFORMATION_MESSAGE);
      resIndex=0;
      seqIndex=0;
    }

    // if allResults is null, this effectively switches displaySearch flag in seqCanvas
    ap.highlightSearchResults( allResults );

    if (findAll)
    {
      String message =  (searchResults.size()/3) + " matches found.";
      JOptionPane.showInternalMessageDialog(this, message, null,
                                            JOptionPane.INFORMATION_MESSAGE);
    }

  }

}
